package lazabs.cfg

import lazabs.ast.ASTree._

/**
 * The transitions in the control flow graph consists of assumes, havocs, and assignments 
 */
sealed abstract class Label
case class Assume(p: Expression) extends Label
case class Assign(lhs: Expression, rhs: Expression) extends Label
case class Havoc(v: Variable) extends Label

/**
 * control flow graph
 */
case class CFGVertex(id: Int) {
  def getId = id
}
case class CFGEdge(label: Label, to: CFGVertex)

object GraphOps {
  type Graph = Map[CFGVertex,Set[CFGEdge]]
  def mergeGraphs(g1:Graph, g2:Graph) : Graph = {
    val keys = g1.keySet ++ g2.keySet
    val g = for {
      k <- keys
      set1 = g1.getOrElse(k,Set[CFGEdge]())
      set2 = g2.getOrElse(k,Set[CFGEdge]())
    } yield (k -> (set1 ++ set2))
    g.toMap
  }
}

case class CFG(start: CFGVertex, transitions: GraphOps.Graph, variables: Set[String], asserts: Map[CFGVertex,Expression])

object ErrorVertex extends CFGVertex(-1)

object CFG {
  def singletonCFG(v:CFGVertex):CFG = CFG(v, Map(), Set(), Map())
}
