package lazabs.viewer

import lazabs.ast.ASTree._
import lazabs.cfg._

object DrawGraph {
  val dotFileName = "DotOutput"
  var fileId = 0
  //Unique IDs for annotation nodes
  var annotId = -1
  def getAnnotId = {annotId += 1; "a" + annotId}

  def apply(cfg: CFG): Unit = {
    val runTime = Runtime.getRuntime   
    val dotOutput = new java.io.FileWriter(dotFileName + fileId + ".dot")
    dotOutput.write( "digraph lazabs {\n")
    dotOutput.write( CFGtoDot(cfg.transitions, cfg.asserts) )
    dotOutput.write( "}")
    dotOutput.close
    var proc = runTime.exec( "dot -Tpng " + "DotOutput" + fileId + ".dot" + " -o graph" + fileId + ".png" )
    proc.waitFor
    //proc = runTime.exec( "eog graph" + fileId + ".png")
    //proc.waitFor
    fileId = fileId + 1
  }
  
  def CFGtoDot(graph: Map[CFGVertex,Set[CFGEdge]], asserts:Map[CFGVertex,Expression]): String = {
    var dotContent = ""
    graph.foreach { case (CFGVertex(id),edges) =>
      edges.foreach { e => 
        dotContent += id + "[label=\"" + id + "\"];\n"
        dotContent += id + " -> " + e.to.id.toString + "[label=\"" + lazabs.viewer.ScalaPrinter(e.label) + "\"];\n"
      }
    }
    def addAnnot(id:Int, e:Expression, shape:String) = {
      val annotId = getAnnotId
      dotContent += annotId + "[label=\"" + lazabs.viewer.ScalaPrinter(e) + "\",shape=" + shape + "];\n"
      dotContent += annotId + " -> " + id + "[style=dotted];\n"
    }
    asserts.foreach { case (CFGVertex(id),e) => addAnnot(id, e, "box")}

    dotContent
  }
}
